unit ContextMenu;

interface

uses Windows, Messages, ShellAPI, ActiveX, ShlObj, NLib;

type
  TContextMenu = class(TObject)
    Parent: HWND;
    CM2: IContextMenu2;
    constructor Create(AParent: HWND; var Menu: HMenu; const Path: string);
    destructor Destroy; override;
    procedure Invoke(Id: Integer);
    procedure HandleMenuMsg(var Message);
  end;

implementation

constructor TContextMenu.Create(AParent: HWND; var Menu: HMenu;
  const Path: string);
var
  ma: IMalloc;
  dir, fname: WideString;
  desktop, folder: IShellFolder;
  len, attr: ULONG;
  dpidl, pidl: PItemIdList;
  cm: IContextMenu;
  r: HRESULT;
begin
  Parent := AParent;
  dpidl := nil;
  pidl := nil;
  dir := DirName(path);
  fname := BaseName(path);
  //SHGetMalloc(ma);
  CoGetMalloc(1, ma);
  SHGetDesktopFolder(desktop);
  try
    r := desktop.ParseDisplayName(0, nil, PWideChar(dir), len, dpidl, attr);
    if FAILED(r) then Exit;
    r := desktop.BindToObject(dpidl, nil, IID_IShellFolder, pointer(folder));
    if FAILED(r) then Exit;
    r := folder.ParseDisplayName(0, nil, PWideChar(fname), len, pidl, attr);
    if FAILED(r) then Exit;
    folder.GetUIObjectOf(Parent, 1, pidl, IID_IContextMenu, nil, pointer(cm));
    CM2 := cm as IContextMenu2;
    CM2.QueryContextMenu(Menu, 0, 1, $7fff, CMF_EXPLORE);
  finally
    ma.Free(dpidl);
    ma.Free(pidl);
  end;
end;

destructor TContextMenu.Destroy;
begin
  CM2 := nil;
end;

procedure TContextMenu.Invoke(Id: Integer);
var
  info: TCMInvokeCommandInfo;
begin
  if Id = 0 then
    Exit;
  FillChar(info, SizeOf(Info), 0);
  info.cbSize := SizeOf(info);
  info.lpVerb := MAKEINTRESOURCE(id-1);
  info.nShow := SW_SHOWNORMAL;
  info.hWnd := Parent;
  CM2.InvokeCommand(info);
end;

procedure TContextMenu.HandleMenuMsg(var Message);
begin
  if Assigned(CM2) then
    with TMessage(Message) do
      case Msg Of
        WM_INITMENUPOPUP, WM_DRAWITEM, WM_MEASUREITEM:
          CM2.HandleMenuMsg(Msg, wParam, lParam);
      end;
end;

initialization
  OleInitialize(nil);

finalization
  OleUnInitialize;

end.
