unit InfoDialog;

interface

uses NTypes, NLib, NForm, NCanvas, NkvImage;

type
  TInfoDialog = class(TDialog)
    Icon: TIcon;
    Path: string;
    PathInArchive: string;
    Image: TNkVImage;
    ATime, MTime: Int64;
    destructor Destroy; override;
    procedure Init(var DefaultFocus: TWindow); override;
    procedure UpdateInfo;
  end;

implementation

{$I dialogid.inc}

function BytesFormat(Value: Integer): string;
var
  s: string;
begin
  Result := UnitFormat(Value);
  s := Copy(Result, 1, Length(Result)-1);
  case Result[Length(Result)] of
    'G': Result := s + ' GB';
    'M': Result := s + ' MB';
    'K': Result := s + ' KB';
  else
    Result := Result + ' bytes';
  end;
end;

destructor TInfoDialog.Destroy;
begin
  if Assigned(Icon) then
    Icon.Free;
  inherited;
end;

procedure TInfoDialog.Init(var DefaultFocus: TWindow);
begin
  UpdateInfo;
end;

procedure TInfoDialog.UpdateInfo;
var
  t: TTime;
begin
  if Path = '' then
  begin
    Items[IDC_ICON].Hide;
    Items[IDC_FILENAME].SetText('<STDIN>')
  end else
  begin
    Items[IDC_ICON].Show;
    if not Assigned(Icon) then
      Icon := TIcon.Create;
    Icon.LoadFileIcon(Path);
    SetIcon(IDC_ICON, Icon);
    if PathInArchive = '' then
      Items[IDC_FILENAME].SetText(BaseName(Path))
    else
      Items[IDC_FILENAME].SetText(PathInArchive)
  end;
  if PathInArchive = '' then
    Items[IDC_LOCATION].SetText(DirName(Path))
  else
    Items[IDC_LOCATION].SetText(Path);
  Items[IDC_SIZE].SetText(Format('{0}x{1}', [Image.Width, Image.Height]));
  Items[IDC_BPP].SetText(Format('{0} bits/pixel', [Image.Bpp]));
  Items[IDC_FORMAT].SetText(Image.FormatName);
  Items[IDC_FILESIZE].SetText(Format('{0} ({1} bytes)',
      [BytesFormat(Image.OrgSize), NumFormat(Image.OrgSize)]));
  t := LocalTime(MTime);
  Items[IDC_MODIFIED].SetText(
      Format('{0:04}-{1:02}-{2:02} {3:02}:{4:02}:{5:02}',
        [t.Year, t.Mon, t.MDay, t.Hour, t.Min, t.Sec]));
  if PathInArchive = '' then
  begin
    t := LocalTime(ATime);
    Items[IDC_ACCESSED].SetText(
        Format('{0:04}-{1:02}-{2:02} {3:02}:{4:02}:{5:02}',
          [t.Year, t.Mon, t.MDay, t.Hour, t.Min, t.Sec]));
  end else
    Items[IDC_ACCESSED].SetText('');
  Items[IDC_MEMORY].SetText(
      Format('{0} ({1} bytes)',
        [BytesFormat(Image.BitsSize), NumFormat(Image.BitsSize)]));
  Items[IDC_PLUGIN].SetText(Image.FilterName);
  Items[IDC_PLUGINFILE].SetText(Image.FilterFilename);
  Items[IDC_TEXTINFOLABEL].Enable(Image.Comment <> '');
  Items[IDC_TEXTINFO].SetText(Image.Comment);
end;

end.

